--local baseObjects = require( modulePath .. "baseObjects")

allObjects = {} -- a list of all objects for us to update and draw.
showPivots = true
uOrder = function(a, b) return a.u > b.u end -- to update objects in correct order.
zOrder = function(a, b) return a.z > b.z end -- to draw objects in correct order.
Drawable = { -- an 'object' more or less styled after the one in the Lua manual.
	new = function (template, self)
		template.init(template, self)
		template.__index = template
		setmetatable(self, template)
		return self
	end,

	init = function (template, self)
		self.name = self.name or template.name or 'head'
		self.sprite = love.graphics.newImage('images/'..self.name..'.png')
		self.w, self.h = self.sprite:getDimensions()
		self.halfw, self.halfh = self.w/2, self.h/2
		self.x, self.y = self.x or template.x or 0, self.y or template.y or 0
		self.u, self.z = self.u or template.u or 0, self.z or template.z or 0
		self.angle = self.angle or template.angle or 0
		self.trueAngle = self.trueAngle or template.trueAngle or self.angle
		self.offset = self.offset or template.offset or {x = 200, y = 0}
		self.pivot = self.pivot or template.pivot or {x = 0, y = 0}
	end,

	update = function (self, dt)
		if self.parent ~= nil then
			local a = math.rad(self.parent.angle)
			xoff = self.offset.y * math.sin(a) - self.offset.x * math.cos(a)
			self.x = self.parent.x - xoff
			yoff = self.offset.y * math.cos(a) + self.offset.x * math.sin(a)
			self.y = self.parent.y + yoff

			self.angle = self.parent.angle + self.trueAngle
		else
			self.angle = self.trueAngle
		end
	end,

	draw = function (self)
		love.graphics.draw(
			self.sprite, self.x, self.y,
			math.rad(self.angle), 1, 1,
			self.halfw+self.pivot.x, self.halfh+self.pivot.y)
	end,

	drawPivots = function (self)
		love.graphics.setColor(255,255,255)
		love.graphics.rectangle('fill',self.x-2,self.y-2,5,5)
		love.graphics.setColor(255,20,20)
		love.graphics.rectangle('fill',self.x-1,self.y-1,3,3)
	end,
}

function love.load()
	love.graphics.setBackgroundColor(255,255,255)
	body = Drawable:new{x = 200, y = 200, angle = 0, name = 'body'}
	head = Drawable:new{x = 200, y = 200, angle = 0, z = 1, parent = body, name = 'head',
		offset = {x = -5, y = -70}, pivot = {x = 0, y = 20}}
	frontarm = Drawable:new{x = 200, y = 200, angle = 0, z = 2, parent = body, name = 'frontarm',
		offset = {x = 15, y = -27}, pivot = {x = -17, y = -2}}
	wand = Drawable:new{x = 200, y = 200, angle = 0, z = 3, parent = frontarm, name = 'wand',
		offset = {x = 37, y = -4}, pivot = {x = -11, y = 30}}
	backarm = Drawable:new{x = 200, y = 200, angle = 0, z = -1, parent = body, name = 'backarm',
		offset = {x = -35, y = -35}, pivot = {x = 10, y = -15}}
	table.insert(allObjects, body)
	table.insert(allObjects, head)
	table.insert(allObjects, frontarm)
	table.insert(allObjects, wand)
	table.insert(allObjects, backarm)
	activeObjectIndex = 2
	activeObject = allObjects[activeObjectIndex]
end

function love.update(dt)
	table.sort(allObjects, uOrder) -- sort objects, then update them.
  for i=#allObjects,1,-1 do allObjects[i]:update(dt) end
end

function love.draw(dt)
	love.graphics.setColor(255,255,255)
	table.sort(allObjects, zOrder) -- sort objects, then draw them.
	for i=#allObjects,1,-1 do allObjects[i]:draw() end
	if showPivots then
		for i=#allObjects,1,-1 do allObjects[i]:drawPivots() end
	end
	love.graphics.setColor(0,0,255)
	love.graphics.print('Active Object: '..activeObject.name)
end

function love.keypressed( k, s )
  if k == "escape" then love.event.quit()
	elseif k == "f12" then showPivots = not showPivots
	elseif k == 'q' then activeObject.trueAngle = activeObject.trueAngle - 22.5
	elseif k == 'e' then activeObject.trueAngle = activeObject.trueAngle + 22.5
	elseif k == 'a' then activeObject.offset.x = activeObject.offset.x - 10
	elseif k == 'd' then activeObject.offset.x = activeObject.offset.x + 10
	elseif k == 'w' then activeObject.offset.y = activeObject.offset.y - 10
	elseif k == 's' then activeObject.offset.y = activeObject.offset.y + 10
	elseif k == 'left' then activeObjectIndex = activeObjectIndex - 1
	elseif k == 'right' then activeObjectIndex = activeObjectIndex + 1
	end
	if activeObjectIndex < 1 then
		activeObjectIndex = activeObjectIndex + 5
	elseif activeObjectIndex > 5 then
		activeObjectIndex = activeObjectIndex % 5
	end
	activeObject = allObjects[activeObjectIndex]
end
